#!/bin/sh

# Setup test keys and certificate in a SoftHSM device.
# Sign a file and verify the signature to check matching keys.

function setup {
    export TESTDIR=`mktemp -d`

    # Create temporary directories
    mkdir -p $TESTDIR
    mkdir -p $TESTDIR/tokens
    mkdir -p $TESTDIR/db

    # Create SoftHSM device
    cat >$TESTDIR/softhsm.conf <<EOF
directories.tokendir = $TESTDIR/db
objectstore.backend = file
EOF

    # Initialize token
    export SOFTHSM2_CONF=$TESTDIR/softhsm.conf
    softhsm2-util --init-token --label softhsm --free --pin 1234 --so-pin 1234 

    # Import private key
    p11tool --provider /usr/lib64/pkcs11/libsofthsm2.so --write \
        --load-privkey "./key.pem" --label test \
        --login --set-pin=1234 "pkcs11:token=softhsm"

    # Import certificate
    p11tool --provider /usr/lib64/pkcs11/libsofthsm2.so --write \
        --load-certificate "./cert.pem" --label test \
        --login --set-pin=1234 "pkcs11:token=softhsm"

    # Extract public key from the certificate
    openssl x509 -pubkey -noout -in "./cert.pem" > "$TESTDIR/pubkey.pem"

    # Import public key
    p11tool --provider /usr/lib64/pkcs11/libsofthsm2.so --write \
        --load-pubkey "$TESTDIR/pubkey.pem" --label test \
        --login --set-pin=1234 "pkcs11:token=softhsm"

    # List objects
    p11tool --list-all --login "pkcs11:token=softhsm" \
        --set-pin=1234

    echo Done!
}

# Cleanup
function teardown {
    rm -rf $TESTDIR
    unset TESTDIR
    unset SOFTHSM2_CONF
}

# Initialize token and import objects
setup

# Generate a file to be signed
echo "secret" > "$TESTDIR/in.txt"

# Sign the file
openssl dgst -sha256 -engine pkcs11 -keyform engine \
    -sign "pkcs11:token=softhsm;object=test;type=private?pin-value=1234" \
    -out "$TESTDIR/signature" "$TESTDIR/in.txt"

# Verify the signature
openssl dgst -engine pkcs11 -keyform engine \
    -verify "pkcs11:token=softhsm;object=test;type=public?pin-value=1234" \
    -signature "$TESTDIR/signature" "$TESTDIR/in.txt"

# Cleanup
teardown
